/**
 *
 * Openutils testing4web (http://www.openmindlab.com/lab/products/testing4web.html)
 * Copyright(C) 2008-2010, Openmind S.r.l. http://www.openmindonline.it
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package net.sourceforge.openutils.testing4web;

import java.io.File;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLDecoder;
import java.util.Hashtable;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.meterware.httpunit.GetMethodWebRequest;
import com.meterware.httpunit.HttpUnitOptions;
import com.meterware.httpunit.WebRequest;
import com.meterware.httpunit.WebResponse;
import com.meterware.servletunit.ServletRunner;


/**
 * Base TestCase class for jsp/web tests. Developers should not use this class directly, but extends the junit or Testng
 * implementation, according to their preferences.
 * @author fgiust
 */
public abstract class BaseJspEmbeddedTest
{

    /**
     * Logger.
     */
    protected Logger log = LoggerFactory.getLogger(getClass());

    /**
     * HttpUnit ServletRunner.
     */
    protected ServletRunner runner;

    /**
     * The context path (by default "/context"
     * @return the context path
     */
    protected String getContext()
    {
        return "/context";
    }

    /**
     * Returns the path to a test web.xml available in the classpath, by default "WEB-INF/web.xml"
     * @return path to a test web.xml available in the classpath
     */
    protected String getWebXmlPathInClasspath()
    {
        return "WEB-INF/web.xml";
    }

    /**
     * Returns the url used for accessing a jsp (defaults to "http://localhost" + getContext() + "/jsps/" + jsp)
     * @param jsp jsp filename
     * @return full url that can be used for servletunit requests
     */
    protected String getJspUrl(String jsp)
    {
        return "http://localhost" + getContext() + "/jsps/" + jsp;
    }

    /**
     * Setup Servletunit
     * @throws Exception
     */
    public void setUp() throws Exception
    {
        File webxmlFile = getWebXml();
        if (!webxmlFile.exists())
        {
            throw new AssertionError("No " + getWebXmlPathInClasspath() + " found in classpath");
        }

        HttpUnitOptions.setDefaultCharacterSet("utf-8");
        System.setProperty("file.encoding", "utf-8");

        // start servletRunner
        runner = new ServletRunner(webxmlFile, getContext());

        Hashtable<String, String> params = new Hashtable<String, String>();
        params.put("javaEncoding", "utf-8");
        params.put("development", "true");
        params.put("keepgenerated", "false");
        params.put("modificationTestInterval", "1000");
        params.put("engineOptionsClass", TestServletOptions.class.getName());

        runner.registerServlet("*.jsp", "org.apache.jasper.servlet.JspServlet", params);

        log.debug("ServletRunner setup OK");
    }

    /**
     * Returns the web.xml file, by loading the path specified by {@link #getWebXmlPathInClasspath()} from the classpath
     * @return web.xml file
     */
    protected File getWebXml()
    {
        // need to pass a web.xml file to setup servletunit working directory
        ClassLoader classLoader = getClass().getClassLoader();
        URL webXmlUrl = classLoader.getResource(getWebXmlPathInClasspath());
        String path = null;
        try
        {
            path = URLDecoder.decode(webXmlUrl.getFile(), "UTF-8");
        }
        catch (UnsupportedEncodingException e)
        {
            // can't happen
        }
        File webxmlFile = new File(path);
        return webxmlFile;
    }

    /**
     * Tear down ServletUnit
     */
    public void tearDown()
    {
        // shutdown servlet engine
        runner.shutDown();
    }

    /**
     * Do a get request to the specified jsp and compare the results with a txt file found in the classpath (path
     * "/jsps/<em>jspname.txt</em>"
     * @param jspname jsp file name
     * @throws Exception any exception thrown by servletunit
     */
    protected void checkGetWithExpectedTxt(String jspname) throws Exception
    {
        WebRequest request = new GetMethodWebRequest(getJspUrl(jspname));

        WebResponse response = runner.getResponse(request);

        if (log.isDebugEnabled())
        {
            log.debug("RESPONSE: " + response.getText());
        }

        String responseText = response.getText();

        String expectedFileName = StringUtils.substringBeforeLast(jspname, ".") + ".txt";

        InputStream stream = getClass().getResourceAsStream("/jsps/" + expectedFileName);
        String expected = IOUtils.toString(stream);
        IOUtils.closeQuietly(stream);

        assertEquals(responseText, expected);
    }

    /**
     * A generic assertEquals method that redirect to the appropriate junit/testng implementation.
     * @param expected expected value
     * @param actual actual value
     */
    protected abstract void assertEquals(String expected, String actual);

}